# NTP server list from: RITM0096493
%define ntpservers 131.225.8.126 131.225.8.127 131.225.17.126 131.225.17.127 131.225.32.205

# set to True if you want to enable the broadcast client
# See: RITM0096493 for current status of this setting
%define usebroadcastclient False

#if SLF5 or SLF6
# these macros are used below to help make future stuff
# easy to port from one thing to another
%define service ntpd
%define site fermi
%define firewallservice %{service}.%{site}
#endif SLF5 or SLF6

################################################################
Summary: This will configure the ntpd config files for Fermilab. 
Name: zz_ntp_configure
Version: 4.2.6
Release: 7.slf%{?rhel}
BuildArch: noarch
License: GPL
Group: Fermi
URL: http://fermilinux.fnal.gov/
Vendor: Fermilab
Packager: Pat Riehecky

Source0: lsb_release
Source1: %{firewallservice}.slf5
Source2: %{firewallservice}.slf6

BuildRequires:  coreutils

Requires(post): coreutils, policycoreutils, bash, augeas, gawk

%if 0%{rhel} < 7
Requires: ntp
Requires(post): chkconfig
%else
# When this package is targeted at SLF7 it expects
#  - the ntpd.service preset has already been loaded
#  - the firewalld default with the ntp port open if using broadcast client
#
# need a statement here, remove when this is actually tested
# In theory we can use chrony or ntpd, chrony is "better"
# Need to look into chrony on 7
Requires: some-slf7-rpm-named-here

# macros exclude things for the 7 package
# don't let anyone try to load you on older things
# exclude RHEL8 for now, no reason to start guessing over there
Requires: system-release > 6
Requires: system-release < 8
%endif

%description

This package configures the ntp time daemon correctly for use at Fermilab.  It
sets the initial time from the default time servers and configures ntpd to
listen to time broadcasts from the local router of the subdomain that the
machine is on.

At this time the default time servers are listed as:
%{ntpservers}

%prep
rm -rf $RPM_BUILD_ROOT

%build
for system in %{ntpservers}; do
    echo ${system} >> FNAL_ntp_servers
done

%install
mkdir -m 755 -p $RPM_BUILD_ROOT/usr/share/doc/%{name}-%{version}/
install -m 644 FNAL_ntp_servers $RPM_BUILD_ROOT/%{_docdir}/%{name}-%{version}/
%if 0%{rhel} < 7
install -m 755 $RPM_SOURCE_DIR/lsb_release $RPM_BUILD_ROOT/%{_docdir}/%{name}-%{version}/
install -m 755 $RPM_SOURCE_DIR/ntpd.fermi.slf5 $RPM_BUILD_ROOT/%{_docdir}/%{name}-%{version}/
install -m 755 $RPM_SOURCE_DIR/ntpd.fermi.slf6 $RPM_BUILD_ROOT/%{_docdir}/%{name}-%{version}/
%endif

%files
%{_docdir}/%{name}-%{version}/

#########################################################################
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
#                            POST/PRE
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
#########################################################################
%post -p /bin/bash
# If the first argument to %pre is 1, the RPM operation is an initial installation.
# If the argument to %pre is 2, the operation is an upgrade from an existing version to a new one. 

####################################################
#### NTPD POST SECTION
####################################################
# Save the config files!
if [[ -f /etc/ntp.conf ]]; then
    if [[ ! -f /etc/ntp.conf.rpmsave ]]; then
        cp -pa /etc/ntp.conf /etc/ntp.conf.rpmsave
    fi

##
%if "%{rhel}" == "5"
    # per RITM0088723
    #   NTP clients shouldn't allow queries from 'just any' FNAL host
    #   only meaningfully effects SLF5, some SLF6 systems have a sufficiently
    #   restrictive entry.
    grep -v '#' /etc/ntp.conf | grep 'restrict 131.225.0.0' 2>&1 >/dev/null
    if [[ $? -eq 0 ]]; then
        TMPFILE=`mktemp`
        cat > $TMPFILE <<RMFNALQ
rm /files/etc/ntp.conf/restrict[ . = '131.225.0.0' ]

save
RMFNALQ
        augtool < $TMPFILE >/dev/null 2>&1
        rm -f $TMPFILE
    fi
%endif
##

    # don't use TUV's ntp servers, we have our own
    grep -v '#' /etc/ntp.conf | grep 'rhel.pool.ntp.org' 2>&1 >/dev/null
    if [[ $? -eq 0 ]]; then
        TMPFILE=`mktemp`
        cat > $TMPFILE <<RMTUVNTPSERVERS
rm /files/etc/ntp.conf/server[ . =~ regexp(".*rhel.pool.ntp.org")]

save
RMTUVNTPSERVERS
        augtool < $TMPFILE >/dev/null 2>&1
        rm -f $TMPFILE
    fi

    # we want honesty rather than calculated expectations, don't fudge the numbers
    grep -v '#' /etc/ntp.conf | grep 'fudge' 2>&1 >/dev/null
    if [[ $? -eq 0 ]]; then
        TMPFILE=`mktemp`
        cat > $TMPFILE <<RMFUDGE
rm /files/etc/ntp.conf/fudge
save
RMFUDGE
        augtool < $TMPFILE >/dev/null 2>&1
        rm -f $TMPFILE
    fi

    # enable checking our NTP servers
    for system in %{ntpservers}; do
        # This is a super expensive way to do this, but it is easy to write!
        grep $system /etc/ntp.conf >/dev/null 2>&1
        if [[ $? -ne 0 ]]; then
            TMPFILE=`mktemp`
            cat > $TMPFILE <<SETSERVER
set /files/etc/ntp.conf/server[99999] $system

save
SETSERVER
            augtool < $TMPFILE >/dev/null 2>&1
            rm -f $TMPFILE
        fi
    done

%if "%{usebroadcastclient}" == "True"
    # enable NTP broadcast client
    echo "I'm unfinished, need to add before broadcastclient to ntp.conf"
    echo "disable auth"
    echo "restrict 131.225.0.0 mask 255.255.0.0 kod nomodify noquery notrap"
    exit 1
    TMPFILE=`mktemp`
    cat > $TMPFILE <<SETBROADCAST
set /files/etc/ntp.conf/broadcastclient

save
SETBROADCAST
    augtool < $TMPFILE >/dev/null 2>&1
    rm -f $TMPFILE
%else
    TMPFILE=`mktemp`
    cat > $TMPFILE <<UNSETBROADCAST
rm /files/etc/ntp.conf/broadcastclient

save
UNSETBROADCAST
    augtool < $TMPFILE >/dev/null 2>&1
    rm -f $TMPFILE
%endif

    restorecon /etc/ntp.conf >/dev/null 2>&1

%if 0%{rhel} > 6
    echo "FINISH ME!"
    exit 1
    # ?
    # ?
    systemctl reload-or-try-restart ntpd.service >/dev/null 2>&1
    systemctl reload-or-try-restart chrony.service >/dev/null 2>&1

%else
    # sometimes you can't restart ntp yet - like in anaconda
    service ntpd restart >/dev/null 2>&1 || true
%endif

fi

####################################################
#### NTPD POST SECTION ENDS
####################################################

####################################################
#### CHRONY POST SECTION
####################################################
# Save the config files!
if [[ -f /etc/chrony.conf ]]; then
    if [[ ! -f /etc/chrony.conf.rpmsave ]]; then
        cp -pa /etc/chrony.conf /etc/chrony.conf.rpmsave
    fi
    restorecon /etc/chrony.conf >/dev/null 2>&1
    echo "I'm unfinished"
    exit 1
fi
####################################################
#### CHRONY POST SECTION ENDS
####################################################

####################################################
#### NTPDATE POST SECTION
####################################################
# This correctly configures /etc/ntp/step-tickers
#  which ntpdate checks for a quick list of systems
#  if you don't tell it where to look

mkdir -p /etc/ntp/
restorecon /etc/ntp >/dev/null 2>&1
touch /etc/ntp/step-tickers >/dev/null 2>&1
for system in %{ntpservers}; do
    grep ${system} /etc/ntp/step-tickers >/dev/null 2>&1
    if [[ $? -ne 0 ]]; then
        echo ${system} >> /etc/ntp/step-tickers
    fi
done

restorecon /etc/ntp/step-tickers >/dev/null 2>&1
####################################################
#### NTPDATE POST SECTION END
####################################################

%preun
# If the first argument to %preun and %postun is 1, the action is an upgrade.
# If the first argument to %preun and %postun is 0, the action is uninstallation. 
%if 0%{rhel} < 7
if [ "$1" = "0" ]; then
    rm -f /etc/sysconfig/%{firewallservice}
fi
%endif

#########################################################################
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
#                            Triggers
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
#########################################################################
%triggerin -p /bin/bash -- ntp
UPGRADING_THE_RPM=0
if [[ $1 -ne 1 ]]; then
    # upgrading %{name}
    UPGRADING_THE_RPM=1
fi

UPGRADING_THE_TRIGGER_RPM=0
if [[ $2 -ne 1 ]]; then
    # upgrading what we trigger on
    UPGRADING_THE_TRIGGER_RPM=1
fi

%if 0%{rhel} > 6

echo "FINISH ME!"
exit 1
# ?
# ?
systemctl reload-or-try-restart ntpd.service >/dev/null 2>&1
systemctl reload-or-try-restart chrony.service >/dev/null 2>&1

%else
# for releases before SLF7, the ntp config is complex, so we need to be sure
# all the moving parts have been put back where they belong

if [[ ! -f /etc/sysconfig/%{firewallservice} ]]; then
    # This copies in the correct firewall script for your version of SLF
    # if somehow it was removed after %post, this will fix it back up...
    VER=$(/usr/share/doc/%{name}-%{version}/lsb_release -r | awk '{ print $2}' | cut -d'.' -f1)
    cp -f /usr/share/doc/%{name}-%{version}/%{firewallservice}.slf$VER /etc/sysconfig/%{firewallservice}
fi

#
# This modifies /etc/init.d/%{service} to poke holes in the firewall for ntp broadcasts
#
if [[ "$(grep /etc/sysconfig/%{firewallservice} /etc/init.d/%{service})" == "" ]] ; then
  # adds /etc/sysconfig/%{firewallservice} to the execution of /etc/init.d/%{service} start
  cat /etc/init.d/%{service} | sed 's:\(start().*\):\1\nif \[ -x /etc/sysconfig/%{firewallservice} \] ; then \n/etc/sysconfig/%{firewallservice} start\nfi:' > /etc/init.d/%{service}.tmp
  mv -f /etc/init.d/%{service}.tmp /etc/init.d/%{service}

  # adds /etc/sysconfig/%{firewallservice} to the execution of /etc/init.d/%{service} stop
  cat /etc/init.d/%{service} | sed 's:\(stop().*\):\1\nif \[ -x /etc/sysconfig/%{firewallservice} \] ; then \n/etc/sysconfig/%{firewallservice} stop\nfi:' > /etc/init.d/%{service}.tmp
  mv -f /etc/init.d/%{service}.tmp /etc/init.d/%{service}

  chmod 755 /etc/init.d/%{service}
fi

# This makes sure the daemon is added to the runlevels 3, 4, and 5
# and restarts the service
/sbin/chkconfig --add ntpd
/sbin/chkconfig --levels 345 ntpd on
/etc/init.d/ntpd restart > /dev/null 2>&1
%endif

# during anaconda ntpd can't always be started
# so make sure $? is a 'success'
echo > /dev/null

#########################################################################
%triggerin -p /bin/bash -- firstboot
#
# This makes it so that you don't get asked about ntp on the firstboot
# since it is already setup now.
# 
if [[ -s /usr/share/firstboot/modules/date.py ]]; then
	rm -f /usr/share/firstboot/modules/date.py*
fi

#########################################################################
#########################################################################
%changelog
* Thu Apr 17 2014 Pat Riehecky <riehecky@fnal.gov> 4.2.6-7
- New NTP servers
- Prep for SLF7
- Moved somewhat away from triggers so you can customize your config after
- per: RITM0096493 no more default broadcast client

* Tue Feb 18 2014 Pat Riehecky <riehecky@fnal.gov> 4.2.6-6
- removing "restrict 131.225.0.0 mask 255.255.0.0 nomodify noquery"
    per RITM0088723
- switched to augeas based config to try and simplify the transition

* Thu Jan 31 2013 Pat Riehecky <riehecky@fnal.gov> 4.2.6-5
- switched to 'bash' for triggers and post to avoid errors
- updated requires list to get things more in order

* Fri Mar 16 2012 Pat Riehecky <riehecky@fnal.gov> 4.2.6-4
- switched to a define based ntp server list
- updated ntp server list
- made version independent, now works on SLF5 and SLF6
- added comments to help explain the configuration options

* Thu Jun 16 2011 Connie Sieh <csieh@fnal.gov> 4.2.6-3
- Modified ntpd-fermi code from "RH-Firewall-1-INPUT" to "INPUT" as TUV
  Does not use "RH-Firewall-1-INPUT" anymore. 

* Thu May 26 2011 Tyler Parsons <tparsons@fnal.gov> 4.2.6-2
- added secondary and tertiary time servers to ntp.conf (131.225.17.150 and 131.225.8.120)
- changed references to servers from (131.225.8.200 and 131.225.17.200) to (131.225.17.150 and 131.225.8.120)
 
* Mon Aug 16 2010 Troy Dawson <dawson@fnal.gov> 4.2.6-1
- added ntpdate to the requirements
- Changed the step-tickers section to trigger on ntpdate
- Removed the change that added -A to /etc/sysconfig/ntpd

* Mon Jun 14 2010 Troy Dawson <dawson@fnal.gov> 4.2.0-9
- Added noquery to the restrict line

